```shell
Compose默认的模板文件名称为 docker-compose.yml，格式为yaml格式.
version: "3"
services:
  webapp:
    image: examples/web
    ports:
      - "80:80"
    volumes:
      - "/data"

注意每个服务都必须通过 image 指令指定镜像或 build 指令（需要 Dockerfile）等来自动构建生成镜像。如果使用 build 指令，在 Dockerfile 中设置的选项(例如：CMD, EXPOSE, VOLUME, ENV 等) 将会自动被获取，无需在 docker-compose.yml 中重复设置。

3.3.1.build
指定 Dockerfile 所在文件夹的路径（可以是绝对路径，或者相对 docker-compose.yml 文件的路径）。 Compose 将会利用它自动构建这个镜像，然后使用这个镜像。
version: '3'
services:
  webapp:
    build: ./dir

可以使用 context 指令指定 Dockerfile 所在文件夹的路径。使用 dockerfile 指令指定 Dockerfile 文件名,这里的文件名可以自定义，默认文件名为Dockerfile, 如果是默认文件名，可以不用指定dockerfile文件，使用 arg 指令指定构建镜像时的变量。
version: '3'
services:
  webapp:
    build:
      context: ./dir
      dockerfile: Dockerfile-alternate
      args:
        buildno: 1

使用 cache_from 指定构建镜像的缓存
build:
  context: .
  cache_from:
    - alpine:latest
    - corp/web_app:3.14

3.3.2.cap_add, cap_drop
指定容器的内核能力（capacity）分配。
例如，让容器拥有所有能力可以指定为：
cap_add:
  - ALL
去掉 NET_ADMIN 能力可以指定为：
cap_drop:
  - NET_ADMIN
3.3.3.command
覆盖容器启动后默认执行的命令
command: bundle exec thin -p 3000
该命令也可以是一个列表，类似于 dockerfile：
command: ["bundle", "exec", "thin", "-p", "3000"]
3.3.4.configs
config 定义仅在 3.3 版及更高版本的撰写文件格式中受支持
3.3.5.Container_name
指定容器名称。默认将会使用 项目名称_服务名称_序号 这样的格式。
container_name: docker-web-container
注意: 由于 Docker 容器名称必须是唯一的，因此如果指定了自定义名称，则不能将服务扩展到 1 个以上的容器。
使用 docker stack deploy 时的注意事项：在 swarm 模式下部署堆栈container_name时忽略该选项。
3.3.6.deploy
指定与服务部署和运行相关的配置,仅用于 Swarm mode.
3.3.7.devices
设备映射列表。使用与--devicedocker 客户端创建选项相同的格式.
devices:
  - "/dev/ttyUSB0:/dev/ttyUSB0"
使用 docker stack deploy 时的注意事项
在 swarm 模式下部署堆栈devices时忽略该选项
3.3.8.depends_on
解决容器的依赖、启动先后的问题,表示服务之间的依赖关系。服务依赖会导致以下行为：
·docker-compose up按依赖顺序启动服务。在下面的例子中，db和redis在 之前启动web。
·docker-compose up SERVICE自动包含SERVICE的依赖项。在下面的示例中，docker-compose up web还创建并启动db和redis。
·docker-compose stop按依赖顺序停止服务。在以下示例中，web在db和之前停止redis。
version: "3.9"services:
  web:
    build: .
    depends_on:
      - db
      - redis
  redis:
    image: redis
  db:
    image: postgres
注意：web 服务不会等待 redis db 「完全启动」之后才启动
3.3.9.dns
自定义 DNS 服务器。可以是一个值，也可以是一个列表.
dns: 8.8.8.8
dns:
  - 8.8.8.8
  - 114.114.114.114
3.3.10.dns_search
配置 DNS 搜索域。可以是一个值，也可以是一个列表。
dns_search: example.com
dns_search:
  - domain1.example.com
  - domain2.example.com
3.3.11.tmpfs
挂载一个 tmpfs 文件系统到容器。
tmpfs: /runtmpfs:
  - /run
  - /tmp
3.3.12.env_file
从文件中获取环境变量，可以为单独的文件路径或列表。如果通过 docker-compose -f FILE 方式来指定 Compose 模板文件，则 env_file 中变量的路径会基于模板文件路径。如果有变量名称与 environment 指令冲突，则按照惯例，以后者为准。
env_file: .env

env_file:
  - ./common.env
  - ./apps/web.env
  - /opt/secrets.env
环境变量文件中每一行必须符合格式，支持 # 开头的注释行
# common.env: Set development environment
PROG_ENV=development
3.3.13.environment
设置环境变量。你可以使用数组或字典两种格式。只给定名称的变量会自动获取运行 Compose 主机上对应变量的值，可以用来防止泄露不必要的数据。
environment:
  RACK_ENV: development
  SESSION_SECRET:
environment:
  - RACK_ENV=development
  - SESSION_SECRET
如果变量名称或者值中用到 true|false，yes|no 等表达 布尔 (opens new window)含义的词汇，最好放到引号里，避免 YAML 自动解析某些内容为对应的布尔语义。这些特定词汇，包括y|Y|yes|Yes|YES|n|N|no|No|NO|true|True|TRUE|false|False|FALSE|on|On|ON|off|Off|OFF
3.3.14.expose
暴露端口，但不映射到宿主机，只被连接的服务访问。仅可以指定内部端口为参数
expose:
 - "3000"
 - "8000"
3.3.15.external_links
注意：不建议使用该指令。链接到 docker-compose.yml 外部的容器，甚至并非 Compose 管理的外部容器。
external_links:
 - redis_1
 - project_db_1:mysql
 - project_db_1:postgresql
3.3.16.extra_hosts
类似 Docker 中的 --add-host 参数，指定额外的 host 名称映射信息。
extra_hosts:
 - "googledns:8.8.8.8"
 - "dockerhub:52.1.157.61"
会在启动后的服务容器中 /etc/hosts 文件中添加如下两条条目。
8.8.8.8 googledns
52.1.157.61 dockerhub
3.3.17.healthcheck
通过命令检查容器是否健康运行。
healthcheck:
  test: ["CMD", "curl", "-f", "http://localhost"]
  interval: 1m30s
  timeout: 10s
  retries: 3
3.3.18.image
指定为镜像名称或镜像 ID。如果镜像在本地不存在，Compose 将会尝试拉取这个镜像。
image: ubuntu
image: orchardup/postgresql
image: a4bc65fd


3.3.19.labels
为容器添加 Docker 元数据（metadata）信息。例如可以为容器添加辅助说明信息。
labels:
  com.startupteam.description: "webapp for a startup team"
  com.startupteam.department: "devops department"
  com.startupteam.release: "rc3 for v1.0"


3.3.20.links
警告: 该--link标志是 Docker 的遗留功能。它可能最终会被删除。除非您绝对需要继续使用它，否则我们建议您使用 用户定义的网络 来促进两个容器之间的通信，而不是使用--link.
用户定义的网络不支持但您可以使用的一项功能 --link是在容器之间共享环境变量。但是，您可以使用其他机制（例如卷）以更可控的方式在容器之间共享环境变量。
链接到另一个服务中的容器。指定服务名称和链接别名 ( "SERVICE:ALIAS")，或仅指定服务名称。
web:
  links:
    - "db"
    - "db:database"
    - "redis"
链接服务的容器可以通过与别名相同的主机名访问，如果未指定别名，则可以使用服务名称.
使用 docker stack deploy 时的注意事项: 在 swarm 模式下部署堆栈links时忽略该选项
3.3.21.logging
服务的日志记录配置
logging:
  driver: syslog
  options:
    syslog-address: "tcp://192.168.0.42:123"
目前支持三种日志驱动类型。
driver: "json-file"
driver: "syslog"
driver: "none"
只有json-file和journald驱动程序可以直接从docker-compose up和获取日志docker-compose logs。使用任何其他驱动程序不会打印任何日志。
options 配置日志驱动的相关参数。设置最大存储大小和最大文件数使用键值对。
options:
  max-size: "200k"
  max-file: "10"

3.3.22.network_mode
设置网络模式。使用与 docker 客户端--network参数相同的值，加上特殊形式service:[service name]。
network_mode: "bridge"
network_mode: "host"
network_mode: "none"
network_mode: "service:[service name]"
network_mode: "container:[container name/id]"

3.3.23.networks
配置容器连接的网络。
version: "3"services:

  some-service:
    networks:
     - some-network
     - other-network
networks:
  some-network:
  other-network:
配置网络需要通过networks声名。
3.3.24.pid
跟主机系统共享进程命名空间。打开该选项的容器之间，以及容器和宿主机系统之间可以通过进程 ID 来相互访问和操作。
pid: "host"

3.3.25.ports
暴露端口信息。
使用宿主端口：容器端口 (HOST:CONTAINER) 格式，或者仅仅指定容器的端口（宿主将会随机选择端口）都可以.
有以下三种选择：
指定两个端口 ( HOST:CONTAINER)
仅指定容器端口（为主机端口选择了一个临时主机端口）。
指定要绑定到 AND 两个端口的主机 IP 地址（默认为 0.0.0.0，表示所有接口）：( IPADDR:HOSTPORT:CONTAINERPORT)。如果 HOSTPORT 为空（例如127.0.0.1::80），则会选择一个临时端口来绑定到主机上。
ports:
  - "3000"
  - "3000-3005"
  - "8000:8000"
  - "9090-9091:8080-8081"
  - "49100:22"
  - "127.0.0.1:8001:8001"
  - "127.0.0.1:5000-5010:5000-5010"
  - "127.0.0.1::5000"
  - "6060:6060/udp"
  - "12400-12500:1240"
注意：以HOST:CONTAINER格式映射端口时，使用小于 60 的容器端口时可能会遇到错误结果，因为 YAML 将格式xx:yy中的数字解析为 base-60 值。因此，我们建议始终将您的端口映射明确指定为字符串

3.3.26.secrets
存储敏感数据，例如 mysql 服务密码。
version: "3.1"services:
mysql:
  image: mysql
  environment:
    MYSQL_ROOT_PASSWORD_FILE: /run/secrets/db_root_password
  secrets:
    - db_root_password
    - my_other_secret
secrets:
  my_secret:
    file: ./my_secret.txt
  my_other_secret:
    external: true
通过将密码定义到文件中，然后外包引入文件读取密码。

3.3.27.security_opt
指定容器模板标签（label）机制的默认属性（用户、角色、类型、级别等）。例如配置标签的用户名和角色名。
security_opt:
    - label:user:USER
    - label:role:ROLE

3.3.28.stop_signal
设置另一个信号来停止容器。在默认情况下使用的是 SIGTERM 停止容器。
stop_signal: SIGUSR1

3.3.29.sysctls
配置容器内核参数。
sysctls:
  net.core.somaxconn: 1024
  net.ipv4.tcp_syncookies: 0
sysctls:
  - net.core.somaxconn=1024
  - net.ipv4.tcp_syncookies=0

3.3.30.ulimits
指定容器的 ulimits 限制值。
例如，指定最大进程数为 65535，指定文件句柄数为 20000（软限制，应用可以随时修改，不能超过硬限制） 和 40000（系统硬限制，只能 root 用户提高）。
  ulimits:
    nproc: 65535
    nofile:
      soft: 20000
      hard: 40000


3.3.31.volumes
数据卷所挂载路径设置。可以设置为宿主机路径(HOST:CONTAINER)或者数据卷名称(VOLUME:CONTAINER)，并且可以设置访问模式 （HOST:CONTAINER:ro）。
该指令中路径支持相对路径。
volumes:
 - /var/lib/mysql
 - cache/:/tmp/cache
 - ~/configs:/etc/configs/:ro
如果路径为数据卷名称，必须在文件中配置数据卷。
version: "3"
services:
  my_src:
    image: mysql:8.0
    volumes:
      - mysql_data:/var/lib/mysql
volumes:
  mysql_data:  

3.3.32.其它指令
此外，还有包括 domainname, entrypoint, hostname, ipc, mac_address, privileged, read_only, shm_size, restart, stdin_open, tty, user, working_dir 等指令，基本跟 docker run 中对应参数的功能一致。
指定服务容器启动后执行的入口文件。
entrypoint: /code/entrypoint.sh
指定容器中运行应用的用户名。
user: nginx
指定容器中工作目录。
working_dir: /code
指定容器中搜索域名、主机名、mac 地址等。
domainname: your_website.comhostname: testmac_address: 08-00-27-00-0C-0A
允许容器中运行一些特权命令。
privileged: true
指定容器退出后的重启策略为始终重启。该命令对保持服务始终运行十分有效，在生产环境中推荐配置为 always 或者 unless-stopped。
restart: always
以只读模式挂载容器的 root 文件系统，意味着不能对容器内容进行修改。
read_only: true
打开标准输入，可以接受外部输入。
stdin_open: true
模拟一个伪终端。
tty: true


3.3.33.读取变量
Compose 模板文件支持动态读取主机的系统环境变量和当前目录下的 .env 文件中的变量。
例如，下面的 Compose 文件将从运行它的环境中读取变量 ${MONGO_VERSION} 的值，并写入执行的指令中。
version: "3"services:
db:
  image: "mongo:${MONGO_VERSION}"
如果执行 MONGO_VERSION=3.2 docker-compose up 则会启动一个 mongo:3.2 镜像的容器；如果执行 MONGO_VERSION=2.8 docker-compose up 则会启动一个 mongo:2.8 镜像的容器。
若当前目录存在 .env 文件，执行 docker-compose 命令时将从该文件中读取变量。
在当前目录新建 .env 文件并写入以下内容。
# 支持 # 号注释MONGO_VERSION=3.6
执行 docker-compose up 则会启动一个 mongo:3.6 镜像的容器。
```

